 --[[ Copyright (c) 2013 Luis Pais

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution.
]]

function returnRect(obj1)
	 return obj1.x, obj1.y, obj1.width, obj1.height
end

function newRect(x,y,w,h)
	 local newrectangle={}
	 newrectangle.x=x
	 newrectangle.y=y
	 newrectangle.width=w
	 newrectangle.height=h
	 return newrectangle
end
function newLine(...)
	 args={...}
	 if math.floor(#args/2)~=#args/2 then error(debug.getinfo(2).currentline..': Number of vertices must be a multiple of two, is '..#args) end
	 local line={}
	 for i=1,#args do
		 table.insert(line,args[i])
	 end
	 return line
end

function moveRect(obj1,mx,my)
	 mx=mx or obj1.x
	 my=my or obj1.y

	 obj1.x=mx
	 obj1.y=my
end

function translateRect(obj1,tx,ty,speed,dt,debugger)
	 if obj1.x==tx and obj1.y==ty then return true end
	 local tx=tx or obj1.x
	 local ty=ty or obj1.y
	 local dt=dt or 1
	 local did=false
	 --print(obj1.x,obj1.y,tx,ty)
	 if obj1.x < tx then --detects if obj1 is to the left of tx
		 if obj1.x+speed*dt > tx then
			 obj1.x=tx
			 did=true
		 else
			 obj1.x = obj1.x + speed*dt
			 did=true
		 end
	 elseif obj1.x > tx then --detect if it is to the right
		 if obj1.x-speed*dt < tx then
			 obj1.x=tx
			 did=true
		 else
			 obj1.x = obj1.x - speed*dt
			 did=true
		 end
	 end
	 if obj1.y < ty then --detects if ty is under obj1
		 if obj1.y +speed*dt > ty then
			 obj1.y = ty
			 did=true
		 else
			 --print('translated '..debugger..' down')
			 obj1.y = obj1.y + speed*dt
			 did=true
		 end
	 elseif obj1.y > ty then --detects if obj1.y is under ty
		 if obj1.y +speed*dt < ty then
			 obj1.y = ty
			 did=true
		 else
			 --print('translated '..debugger..' up')
			 obj1.y = obj1.y - speed*dt
			 did=true
		 end
	 end
	 if did then return false else return true end
end	  

function resizeRect(obj1,gw,gh)
	 gw=gw or obj1.width
	 gh=gh or obj1.height
	 obj1.width = gw
	 obj1.height = gh
end
 
function reshapeRect(obj1,rx,ry,rw,rh)
	 rx=rx or obj1.x
	 ry=ry or obj1.y
	 rw=rw or obj1.width
	 rh=rh or obj1.height
	 obj1.x=rx
	 obj1.y=ry
	 obj1.width=rw
	 obj1.height=rh
end

function getArea(obj1)
	 return obj1.width*obj1.height
end

function getInter(a, b)
    local left = math.max(a.x, b.x)
    local right = math.min(a.x + a.width, b.x + b.width)
    local top = math.max(a.y, b.y)
    local bottom = math.min(a.y + a.height, b.y + b.height)
    if left < right and top < bottom then
        return newRect(left, top, right - left, bottom - top)
	else
		 return newRect(0,0,0,0)
    end
end

function isInter(obj1,obj2,tolerance)
	 tolerance=tolerance or 0
	 if isEmpty(obj1) or isEmpty(obj2) then
		 return false
	 end
	 if obj1.x+obj1.width-tolerance < obj2.x or obj1.x+tolerance > obj2.x+obj2.width or obj1.y+obj1.height-tolerance < obj2.y or obj1.y-tolerance > obj2.y+obj2.height then
		 return false
	 else
		 return true
	 end
end

function isInterMW(line1,line2)
	 local p1 = {x=line2[1],y=line2[2]}
	 local p2 = {x=line2[#line2-1],y=line2[#line2]}
	 local p3 = {x=line1[1],y=line1[2]}
	 local p4 = {x=line1[#line1-1],y=line1[#line1]}
	 
	 local xD1,yD1,xD2,yD2,xD3,yD3
	 local dot,deg,len1,len2,pt
	 local segmentLen1,segmentLen2
	 local ua,ub,div
	 
	 xD1 = p2.x-p1.x
	 xD2 = p4.x-p3.x
	 yD1 = p2.y-p1.y
	 yD2 = p4.y-p3.y
	 xD3 = p1.x-p3.x
	 yD3 = p1.y-p3.y
	 
	 len1 = math.sqrt(xD1*xD1+yD1*yD1)
	 len2 = math.sqrt(xD2*xD2+yD2*yD2)
	 
	 dot = (xD1*xD2+yD1*yD2)
	 deg = dot/(len1*len2)
	 
	 if math.abs(deg) == 1 then return false end
	 
	 pt = {x = 0, y = 0}
	 div = yD2*xD1-xD2*yD1
	 ua = (xD2*yD3-yD2*xD3)/div
	 ub = (xD1*yD3-yD1*xD3)/div
	 pt.x = p1.x+ua*xD1
	 pt.y = p1.y+ua*yD1
	 
	 xD1 = pt.x-p1.x
	 xD2 = pt.x-p2.x
	 yD1 = pt.y-p1.y
	 yD2 = pt.y-p2.y
	 segmentLen1 = math.sqrt(xD1*xD1+yD1*yD1)+math.sqrt(xD2*xD2+yD2*yD2)
	 
	 xD1 = pt.x-p3.x
	 xD2 = pt.x-p4.x
	 yD1 = pt.y-p3.y
	 yD2 = pt.y-p4.y
	 segmentLen2 = math.sqrt(xD1*xD1+yD1*yD1)+math.sqrt(xD2*xD2+yD2*yD2)
	 
	 if math.abs(len1 - segmentLen1) > 0.01 or math.abs(len2 - segmentLen2)>0.01 then
		 return false
	 end
	 
	 return true, pt
end

function getInterPointCK(line1,line2)
	 local x1=line2[1]
	 local y1=line2[2]
	 local x2=line2[#line2-1]
	 local y2=line2[#line2]
	 local x3=line1[1]
	 local y3=line1[2]
	 local x4=line1[#line1-1]
	 local y4=line1[#line1]
	 
	 local det1And2 = x1*y2 - y1*x2
	 local det3And4 = x3*y4 - y3*x4
     local x1LessX2 = x1 - x2
     local y1LessY2 = y1 - y2
     local x3LessX4 = x3 - x4
     local y3LessY4 = y3 - y4
     local det1Less2And3Less4 = x1LessX2*y3LessY4 - y1LessY2*x3LessX4
	 
	 if det1Less2And3Less4 == 0 then
		 return false
	 end
	 
	 local x = (det1And2*x3LessX4 - x1LessX2*det3And4) / det1Less2And3Less4
	 local y = (det1And2*y3LessY4 - y1LessY2*det3And4) / det1Less2And3Less4
	 
	 return {x=x,y=y}
end

function isIn(obj1,px,py)
	 if px > obj1.x and px < obj1.x+obj1.width and py > obj1.y and py < obj1.y+obj1.height then
		 return true
	 else
		 return false
	 end
end

function isPointInLine(line,point) --http://stackoverflow.com/questions/11907947/how-to-check-if-a-point-lays-on-a-line-between-2-provided-points
	 --This function suffers from floating point precision glitches
	 local dxc = point.x - line[1]
	 local dyc = point.y - line[2]
	 local dxc2 = point.x - line[#line-1]
	 local dyc2 = point.y - line[#line]
	 
	 local dx1 = line[#line-1] - line[1]
	 local dy1 = line[#line] - line[2]
	 
	 local cross = dxc*dy1 - dyc*dx1
	 
	 if cross ~= 0 then return false end -- point is not part of the line segment
	 
	 if math.abs(dx1) >= math.abs(dy1) then 
		 if dx1 > 0 then
			 return (line[1] <= point.x and point.x <= line[#line-1])
		 else
			 return (line[#line-1] <= point.x and point.x <= line[1])
		 end
	 else
		 if dy1 > 0 then
			 return (line[2] <= point.y and point.y <= line[#line])
		 else
			 return (line[#line] <= point.y and point.y <= line[2])
		 end
	 end
	 
	 return (math.sqrt(math.abs(dxc)^2-math.abs(dxc)^2)+math.sqrt(math.abs(dxc2)^2-math.abs(dyc2)^2)==math.sqrt(math.abs(dx1)^2-math.abs(dy1)^2))
end

function isPointInLineLP(point,line,debugd)
	 p={x=point.x or point[1], y=point.y or point[2]}

	 if line[1] == line[#line-1] then --line is vertical
		 if p.y >= line[2] and p.y <= line[#line] then
			 return true
		 end
	 elseif line[2] == line[#line] then --line is horizontal
		 if p.x >= line[1] and p.x <= line[#line-1] then
			 return true
		 end
	 end
	 return false
end

function getUnion(obj1,obj2)
	 local unionRect = {}
	 unionRect.x1=math.min(obj1.x,obj2.x)
	 unionRect.x2=math.max(obj1.x+obj1.width,obj2.x+obj2.width)
	 unionRect.y1=math.min(obj1.y,obj2.y)
	 unionRect.y2=math.max(obj1.y+obj1.height, obj2.y+obj2.height)
	 if not isEmpty(obj1) and not isEmpty(obj2) then
		 return newRect(unionRect.x1,unionRect.y1,unionRect.x2-unionRect.x1,unionRect.y2-unionRect.y1)
	 else
		 return newRect(0,0,0,0)
	 end
end

function equalsRect(obj1,obj2)
	 if obj1.x==obj2.x and obj1.y==obj2.y and obj1.width==obj2.width and obj1.height==obj2.height then
		 return true
	 else
		 return false
	 end
end

function emptyRect(obj1)
	 obj1.x=0
	 obj1.y=0
	 obj1.width=0
	 obj1.height=0
end

function isEmpty(obj1)
	 if (obj1==nil) or (obj1.x==nil and obj1.y==nil and obj1.width==nil and obj1.height==nil) or (obj1.width <=0 and obj1.height <= 0) then
		 return true
	 else
		 return false
	 end
end

function tostringRect(obj1,rectname)
	 if rectname == nil then
		 return 'x='..obj1.x..',y='..obj1.y..',width='..obj1.width..',height='..obj1.height
	 else
		 return rectname..'.x='..obj1.x..', '..rectname..'.y='..obj1.y..', '..rectname..'.width='..obj1.width..', '..rectname..'.height='..obj1.height
	 end
end
